@@ -1053,7 +1053,7 @@ impl<'a> CompletionContext<'a> {
 
     /// A small window of context before the cursor.
     fn before_window(&self, size: usize) -> &str {
-        &self.before[self.cursor.saturating_sub(size)..]
+        Scanner::new(self.before).from(self.cursor.saturating_sub(size))
     }
 
     /// Add a prefix and suffix to all applications.
@@ -1403,3 +1403,42 @@ impl<'a> CompletionContext<'a> {
         }
     }
 }
+
+#[cfg(test)]
+mod tests {
+    use typst::eval::Tracer;
+
+    use super::autocomplete;
+    use crate::tests::TestWorld;
+
+    #[track_caller]
+    fn test(text: &str, cursor: usize, contains: &[&str], excludes: &[&str]) {
+        let world = TestWorld::new(text);
+        let doc = typst::compile(&world, &mut Tracer::new()).ok();
+        let (_, completions) =
+            autocomplete(&world, doc.as_ref(), &world.main, cursor, true)
+                .unwrap_or_default();
+
+        let labels: Vec<_> = completions.iter().map(|c| c.label.as_str()).collect();
+        for item in contains {
+            assert!(labels.contains(item), "{item:?} was not contained in {labels:?}");
+        }
+        for item in excludes {
+            assert!(!labels.contains(item), "{item:?} was not excluded in {labels:?}");
+        }
+    }
+
+    #[test]
+    fn test_autocomplete() {
+        test("#i", 2, &["int", "if conditional"], &["foo"]);
+        test("#().", 4, &["insert", "remove", "len", "all"], &["foo"]);
+    }
+
+    #[test]
+    fn test_before_window_char_boundary() {
+        // Check that the `before_window` doesn't slice into invalid byte
+        // boundaries.
+        let s = "😀😀     #text(font: \"\")";
+        test(s, s.len() - 2, &[], &[]);
+    }
+}
