@@ -229,17 +229,24 @@ fn write_page(ctx: &mut PdfContext, i: usize, resources_ref: Ref) {
 
 /// Write the page labels.
 pub(crate) fn write_page_labels(ctx: &mut PdfContext) -> Vec<(NonZeroUsize, Ref)> {
+    // If there is no page labeled, we skip the writing
+    if !ctx.pages.iter().any(|p| {
+        p.label
+            .as_ref()
+            .is_some_and(|l| l.prefix.is_some() || l.style.is_some())
+    }) {
+        return Vec::new();
+    }
+
     let mut result = vec![];
+    let empty_label = PdfPageLabel::default();
     let mut prev: Option<&PdfPageLabel> = None;
 
     for (i, page) in ctx.pages.iter().enumerate() {
         let nr = NonZeroUsize::new(1 + i).unwrap();
-        let Some(label) = &page.label else { continue };
-
-        // Don't create a label if neither style nor prefix are specified.
-        if label.prefix.is_none() && label.style.is_none() {
-            continue;
-        }
+        // If there are pages with empty labels between labeled pages, we must
+        // write empty PageLabel entries.
+        let label = page.label.as_ref().unwrap_or(&empty_label);
 
         if let Some(pre) = prev {
             if label.prefix == pre.prefix
@@ -316,9 +323,7 @@ impl PdfPageLabel {
             return None;
         };
 
-        let Some((prefix, kind, case)) = pat.pieces.first() else {
-            return None;
-        };
+        let (prefix, kind, case) = pat.pieces.first()?;
 
         // If there is a suffix, we cannot use the common style optimisation,
         // since PDF does not provide a suffix field.
@@ -372,7 +377,7 @@ pub struct EncodedPage {
 }
 
 /// Represents a resource being used in a PDF page by its name.
-#[derive(Debug, Clone, PartialEq, Eq, Hash)]
+#[derive(Debug, Clone, PartialEq, Eq, Hash, Ord, PartialOrd)]
 pub struct PageResource {
     kind: ResourceKind,
     name: EcoString,
@@ -385,7 +390,7 @@ impl PageResource {
 }
 
 /// A kind of resource being used in a PDF page.
-#[derive(Debug, Clone, PartialEq, Eq, Hash)]
+#[derive(Debug, Clone, PartialEq, Eq, Hash, Ord, PartialOrd)]
 pub enum ResourceKind {
     XObject,
     Font,
@@ -829,7 +834,7 @@ fn write_shape(ctx: &mut PageContext, pos: Point, shape: &Shape) {
         Geometry::Rect(size) => {
             let w = size.x.to_f32();
             let h = size.y.to_f32();
-            if w > 0.0 && h > 0.0 {
+            if w.abs() > f32::EPSILON && h.abs() > f32::EPSILON {
                 ctx.content.rect(x, y, w, h);
             }
         }
