@@ -1,6 +1,7 @@
 //! Parser and syntax tree for Typst.
 
 pub mod ast;
+pub mod package;
 
 mod file;
 mod highlight;
@@ -8,11 +9,13 @@ mod kind;
 mod lexer;
 mod node;
 mod parser;
+mod path;
 mod reparser;
+mod set;
 mod source;
 mod span;
 
-pub use self::file::{FileId, PackageSpec, PackageVersion, VirtualPath};
+pub use self::file::FileId;
 pub use self::highlight::{highlight, highlight_html, Tag};
 pub use self::kind::SyntaxKind;
 pub use self::lexer::{
@@ -20,6 +23,7 @@ pub use self::lexer::{
 };
 pub use self::node::{LinkedChildren, LinkedNode, SyntaxError, SyntaxNode};
 pub use self::parser::{parse, parse_code, parse_math};
+pub use self::path::VirtualPath;
 pub use self::source::Source;
 pub use self::span::{Span, Spanned};
 
