@@ -6,8 +6,8 @@ use ecow::eco_format;
 
 use crate::diag::{bail, At, SourceResult, StrResult};
 use crate::eval::{access_dict, Access, Eval, Vm};
-use crate::foundations::{format_str, Datetime, IntoValue, Regex, Repr, Smart, Value};
-use crate::layout::{Align, Length, Rel};
+use crate::foundations::{format_str, Datetime, IntoValue, Regex, Repr, Value};
+use crate::layout::{Alignment, Length, Rel};
 use crate::syntax::ast::{self, AstNode};
 use crate::text::TextElem;
 use crate::util::Numeric;
@@ -16,7 +16,6 @@ use crate::visualize::Stroke;
 impl Eval for ast::Unary<'_> {
     type Output = Value;
 
-    #[tracing::instrument(name = "Unary::eval", skip_all)]
     fn eval(self, vm: &mut Vm) -> SourceResult<Self::Output> {
         let value = self.expr().eval(vm)?;
         let result = match self.op() {
@@ -31,7 +30,6 @@ impl Eval for ast::Unary<'_> {
 impl Eval for ast::Binary<'_> {
     type Output = Value;
 
-    #[tracing::instrument(name = "Binary::eval", skip_all)]
     fn eval(self, vm: &mut Vm) -> SourceResult<Self::Output> {
         match self.op() {
             ast::BinOp::Add => apply_binary(self, vm, add),
@@ -150,7 +148,7 @@ pub fn pos(value: Value) -> StrResult<Value> {
             mismatch!("cannot apply unary '+' to {}", value)
         }
         Dyn(d) => {
-            if d.is::<Align>() {
+            if d.is::<Alignment>() {
                 mismatch!("cannot apply unary '+' to {}", d)
             } else {
                 mismatch!("cannot apply '+' to {}", d)
@@ -219,28 +217,15 @@ pub fn add(lhs: Value, rhs: Value) -> StrResult<Value> {
         (Array(a), Array(b)) => Array(a + b),
         (Dict(a), Dict(b)) => Dict(a + b),
 
-        (Color(color), Length(thickness)) | (Length(thickness), Color(color)) => Stroke {
-            paint: Smart::Custom(color.into()),
-            thickness: Smart::Custom(thickness),
-            ..Stroke::default()
+        (Color(color), Length(thickness)) | (Length(thickness), Color(color)) => {
+            Stroke::from_pair(color, thickness).into_value()
         }
-        .into_value(),
-
         (Gradient(gradient), Length(thickness))
-        | (Length(thickness), Gradient(gradient)) => Stroke {
-            paint: Smart::Custom(gradient.into()),
-            thickness: Smart::Custom(thickness),
-            ..Stroke::default()
+        | (Length(thickness), Gradient(gradient)) => {
+            Stroke::from_pair(gradient, thickness).into_value()
         }
-        .into_value(),
-
         (Pattern(pattern), Length(thickness)) | (Length(thickness), Pattern(pattern)) => {
-            Stroke {
-                paint: Smart::Custom(pattern.into()),
-                thickness: Smart::Custom(thickness),
-                ..Stroke::default()
-            }
-            .into_value()
+            Stroke::from_pair(pattern, thickness).into_value()
         }
 
         (Duration(a), Duration(b)) => Duration(a + b),
@@ -253,7 +238,9 @@ pub fn add(lhs: Value, rhs: Value) -> StrResult<Value> {
 
         (Dyn(a), Dyn(b)) => {
             // Alignments can be summed.
-            if let (Some(&a), Some(&b)) = (a.downcast::<Align>(), b.downcast::<Align>()) {
+            if let (Some(&a), Some(&b)) =
+                (a.downcast::<Alignment>(), b.downcast::<Alignment>())
+            {
                 return Ok((a + b)?.into_value());
             }
 
@@ -551,6 +538,7 @@ pub fn compare(lhs: &Value, rhs: &Value) -> StrResult<Ordering> {
 
         (Duration(a), Duration(b)) => a.cmp(b),
         (Datetime(a), Datetime(b)) => try_cmp_datetimes(a, b)?,
+        (Array(a), Array(b)) => try_cmp_arrays(a.as_slice(), b.as_slice())?,
 
         _ => mismatch!("cannot compare {} and {}", lhs, rhs),
     })
@@ -568,6 +556,26 @@ fn try_cmp_datetimes(a: &Datetime, b: &Datetime) -> StrResult<Ordering> {
         .ok_or_else(|| eco_format!("cannot compare {} and {}", a.kind(), b.kind()))
 }
 
+/// Try to compare arrays of values lexicographically.
+fn try_cmp_arrays(a: &[Value], b: &[Value]) -> StrResult<Ordering> {
+    a.iter()
+        .zip(b.iter())
+        .find_map(|(first, second)| {
+            match compare(first, second) {
+                // Keep searching for a pair of elements that isn't equal.
+                Ok(Ordering::Equal) => None,
+                // Found a pair which either is not equal or not comparable, so
+                // we stop searching.
+                result => Some(result),
+            }
+        })
+        .unwrap_or_else(|| {
+            // The two arrays are equal up to the shortest array's extent,
+            // so compare their lengths instead.
+            Ok(a.len().cmp(&b.len()))
+        })
+}
+
 /// Test whether one value is "in" another one.
 pub fn in_(lhs: Value, rhs: Value) -> StrResult<Value> {
     if let Some(b) = contains(&lhs, &rhs) {
