@@ -1,8 +1,9 @@
 use std::cmp::Ordering;
 use std::fmt::{Debug, Formatter};
-use std::num::NonZeroI64;
+use std::num::{NonZeroI64, NonZeroUsize};
 use std::ops::{Add, AddAssign};
 
+use comemo::Tracked;
 use ecow::{eco_format, EcoString, EcoVec};
 use serde::{Deserialize, Serialize};
 use smallvec::SmallVec;
@@ -11,8 +12,8 @@ use crate::diag::{At, SourceResult, StrResult};
 use crate::engine::Engine;
 use crate::eval::ops;
 use crate::foundations::{
-    cast, func, repr, scope, ty, Args, Bytes, CastInfo, FromValue, Func, IntoValue,
-    Reflect, Repr, Value, Version,
+    cast, func, repr, scope, ty, Args, Bytes, CastInfo, Context, FromValue, Func,
+    IntoValue, Reflect, Repr, Value, Version,
 };
 use crate::syntax::Span;
 
@@ -67,7 +68,7 @@ pub use crate::__array as array;
 /// #(("A", "B", "C")
 ///     .join(", ", last: " and "))
 /// ```
-#[ty(scope)]
+#[ty(scope, cast)]
 #[derive(Default, Clone, PartialEq, Hash, Serialize, Deserialize)]
 #[serde(transparent)]
 pub struct Array(EcoVec<Value>);
@@ -113,7 +114,7 @@ impl Array {
         let len = self.len();
         self.locate_opt(index, false)
             .and_then(move |i| self.0.make_mut().get_mut(i))
-            .ok_or_else(|| out_of_bounds_no_default(index, len))
+            .ok_or_else(|| out_of_bounds(index, len))
     }
 
     /// Resolve an index or throw an out of bounds error.
@@ -300,12 +301,14 @@ impl Array {
         &self,
         /// The engine.
         engine: &mut Engine,
+        /// The callsite context.
+        context: Tracked<Context>,
         /// The function to apply to each item. Must return a boolean.
         searcher: Func,
     ) -> SourceResult<Option<Value>> {
         for item in self.iter() {
             if searcher
-                .call(engine, [item.clone()])?
+                .call(engine, context, [item.clone()])?
                 .cast::<bool>()
                 .at(searcher.span())?
             {
@@ -322,12 +325,14 @@ impl Array {
         &self,
         /// The engine.
         engine: &mut Engine,
+        /// The callsite context.
+        context: Tracked<Context>,
         /// The function to apply to each item. Must return a boolean.
         searcher: Func,
     ) -> SourceResult<Option<i64>> {
         for (i, item) in self.iter().enumerate() {
             if searcher
-                .call(engine, [item.clone()])?
+                .call(engine, context, [item.clone()])?
                 .cast::<bool>()
                 .at(searcher.span())?
             {
@@ -397,12 +402,18 @@ impl Array {
         &self,
         /// The engine.
         engine: &mut Engine,
+        /// The callsite context.
+        context: Tracked<Context>,
         /// The function to apply to each item. Must return a boolean.
         test: Func,
     ) -> SourceResult<Array> {
         let mut kept = EcoVec::new();
         for item in self.iter() {
-            if test.call(engine, [item.clone()])?.cast::<bool>().at(test.span())? {
+            if test
+                .call(engine, context, [item.clone()])?
+                .cast::<bool>()
+                .at(test.span())?
+            {
                 kept.push(item.clone())
             }
         }
@@ -416,10 +427,14 @@ impl Array {
         self,
         /// The engine.
         engine: &mut Engine,
+        /// The callsite context.
+        context: Tracked<Context>,
         /// The function to apply to each item.
         mapper: Func,
     ) -> SourceResult<Array> {
-        self.into_iter().map(|item| mapper.call(engine, [item])).collect()
+        self.into_iter()
+            .map(|item| mapper.call(engine, context, [item]))
+            .collect()
     }
 
     /// Returns a new array with the values alongside their indices.
@@ -521,6 +536,8 @@ impl Array {
         self,
         /// The engine.
         engine: &mut Engine,
+        /// The callsite context.
+        context: Tracked<Context>,
         /// The initial value to start with.
         init: Value,
         /// The folding function. Must have two parameters: One for the
@@ -529,7 +546,7 @@ impl Array {
     ) -> SourceResult<Value> {
         let mut acc = init;
         for item in self {
-            acc = folder.call(engine, [acc, item])?;
+            acc = folder.call(engine, context, [acc, item])?;
         }
         Ok(acc)
     }
@@ -581,11 +598,13 @@ impl Array {
         self,
         /// The engine.
         engine: &mut Engine,
+        /// The callsite context.
+        context: Tracked<Context>,
         /// The function to apply to each item. Must return a boolean.
         test: Func,
     ) -> SourceResult<bool> {
         for item in self {
-            if test.call(engine, [item])?.cast::<bool>().at(test.span())? {
+            if test.call(engine, context, [item])?.cast::<bool>().at(test.span())? {
                 return Ok(true);
             }
         }
@@ -599,11 +618,13 @@ impl Array {
         self,
         /// The engine.
         engine: &mut Engine,
+        /// The callsite context.
+        context: Tracked<Context>,
         /// The function to apply to each item. Must return a boolean.
         test: Func,
     ) -> SourceResult<bool> {
         for item in self {
-            if !test.call(engine, [item])?.cast::<bool>().at(test.span())? {
+            if !test.call(engine, context, [item])?.cast::<bool>().at(test.span())? {
                 return Ok(false);
             }
         }
@@ -704,6 +725,36 @@ impl Array {
         Array(vec)
     }
 
+    /// Splits an array into non-overlapping chunks, starting at the beginning,
+    /// ending with a single remainder chunk.
+    ///
+    /// All chunks but the last have `chunk-size` elements.
+    /// If `exact` is set to `{true}`, the remainder is dropped if it
+    /// contains less than `chunk-size` elements.
+    ///
+    /// ```example
+    /// #let array = (1, 2, 3, 4, 5, 6, 7, 8)
+    /// #array.chunks(3)
+    /// #array.chunks(3, exact: true)
+    /// ```
+    #[func]
+    pub fn chunks(
+        self,
+        /// How many elements each chunk may at most contain.
+        chunk_size: NonZeroUsize,
+        /// Whether to keep the remainder if its size is less than `chunk-size`.
+        #[named]
+        #[default(false)]
+        exact: bool,
+    ) -> Array {
+        let to_array = |chunk| Array::from(chunk).into_value();
+        if exact {
+            self.0.chunks_exact(chunk_size.get()).map(to_array).collect()
+        } else {
+            self.0.chunks(chunk_size.get()).map(to_array).collect()
+        }
+    }
+
     /// Return a sorted version of this array, optionally by a given key
     /// function. The sorting algorithm used is stable.
     ///
@@ -714,6 +765,8 @@ impl Array {
         self,
         /// The engine.
         engine: &mut Engine,
+        /// The callsite context.
+        context: Tracked<Context>,
         /// The callsite span.
         span: Span,
         /// If given, applies this function to the elements in the array to
@@ -726,7 +779,7 @@ impl Array {
         let mut key_of = |x: Value| match &key {
             // NOTE: We are relying on `comemo`'s memoization of function
             // evaluation to not excessively reevaluate the `key`.
-            Some(f) => f.call(engine, [x]),
+            Some(f) => f.call(engine, context, [x]),
             None => Ok(x),
         };
         vec.make_mut().sort_by(|a, b| {
@@ -762,6 +815,8 @@ impl Array {
         self,
         /// The engine.
         engine: &mut Engine,
+        /// The callsite context.
+        context: Tracked<Context>,
         /// If given, applies this function to the elements in the array to
         /// determine the keys to deduplicate by.
         #[named]
@@ -771,7 +826,7 @@ impl Array {
         let mut key_of = |x: Value| match &key {
             // NOTE: We are relying on `comemo`'s memoization of function
             // evaluation to not excessively reevaluate the `key`.
-            Some(f) => f.call(engine, [x]),
+            Some(f) => f.call(engine, context, [x]),
             None => Ok(x),
         };
 
@@ -803,8 +858,8 @@ pub struct ToArray(Array);
 
 cast! {
     ToArray,
-    v: Bytes => Self(v.iter().map(|&b| Value::Int(b.into())).collect()),
     v: Array => Self(v),
+    v: Bytes => Self(v.iter().map(|&b| Value::Int(b.into())).collect()),
     v: Version => Self(v.values().iter().map(|&v| Value::Int(v as i64)).collect())
 }
 
