@@ -321,9 +321,6 @@ trait Blockable: Debug + Send + Sync + 'static {
     /// Equivalent to `downcast_ref` for the block.
     fn as_any(&self) -> &dyn Any;
 
-    /// Equivalent to `downcast_mut` for the block.
-    fn as_any_mut(&mut self) -> &mut dyn Any;
-
     /// Equivalent to [`Hash`] for the block.
     fn dyn_hash(&self, state: &mut dyn Hasher);
 
@@ -336,10 +333,6 @@ impl<T: Debug + Clone + Hash + Send + Sync + 'static> Blockable for T {
         self
     }
 
-    fn as_any_mut(&mut self) -> &mut dyn Any {
-        self
-    }
-
     fn dyn_hash(&self, mut state: &mut dyn Hasher) {
         // Also hash the TypeId since values with different types but
         // equal data should be different.
