@@ -2,7 +2,9 @@ use ecow::EcoString;
 use unicode_segmentation::UnicodeSegmentation;
 
 use crate::diag::{bail, StrResult};
-use crate::foundations::{array, cast, dict, elem, Array, Dict, FromValue, Smart, Str};
+use crate::foundations::{
+    array, cast, dict, elem, Array, Dict, FromValue, Packed, PlainText, Smart, Str,
+};
 use crate::layout::Dir;
 use crate::syntax::is_newline;
 use crate::text::{Lang, Region};
@@ -26,7 +28,7 @@ use crate::text::{Lang, Region};
 /// # Syntax
 /// This function also has dedicated syntax: The normal quote characters
 /// (`'` and `"`). Typst automatically makes your quotes smart.
-#[elem(name = "smartquote")]
+#[elem(name = "smartquote", PlainText)]
 pub struct SmartQuoteElem {
     /// Whether this should be a double quote.
     #[default(true)]
@@ -85,6 +87,16 @@ pub struct SmartQuoteElem {
     pub quotes: Smart<SmartQuoteDict>,
 }
 
+impl PlainText for Packed<SmartQuoteElem> {
+    fn plain_text(&self, text: &mut EcoString) {
+        if self.double.unwrap_or(true) {
+            text.push_str("\"");
+        } else {
+            text.push_str("'");
+        }
+    }
+}
+
 /// State machine for smart quote substitution.
 #[derive(Debug, Clone)]
 pub struct SmartQuoter {
