@@ -12,7 +12,6 @@ use crate::foundations::{
 };
 use crate::layout::{Angle, Axes, Dir, Quadrant, Ratio};
 use crate::syntax::{Span, Spanned};
-use crate::visualize::color::{Hsl, Hsv};
 use crate::visualize::{Color, ColorSpace, WeightedColor};
 
 /// A color gradient.
@@ -72,21 +71,21 @@ use crate::visualize::{Color, ColorSpace, WeightedColor};
 /// stops evenly.
 ///
 /// # Relativeness
-/// The location of the `{0%}` and `{100%}` stops is dependant on the dimensions
-/// of a container. This container can either be the shape they are painted on,
-/// or the closest surrounding container. This is controlled by the `relative`
-/// argument of a gradient constructor. By default, gradients are relative to
-/// the shape they are painted on, unless the gradient is applied on text, in
-/// which case they are relative to the closest ancestor container.
+/// The location of the `{0%}` and `{100%}` stops depends on the dimensions
+/// of a container. This container can either be the shape that it is being
+/// painted on, or the closest surrounding container. This is controlled by the
+/// `relative` argument of a gradient constructor. By default, gradients are
+/// relative to the shape they are being painted on, unless the gradient is
+/// applied on text, in which case they are relative to the closest ancestor
+/// container.
 ///
 /// Typst determines the ancestor container as follows:
 /// - For shapes that are placed at the root/top level of the document, the
 ///   closest ancestor is the page itself.
-/// - For other shapes, the ancestor is the innermost [`block`]($block) or
-///   [`box`]($box) that contains the shape. This includes the boxes and blocks
-///   that are implicitly created by show rules and elements. For example, a
-///   [`rotate`]($rotate) will not affect the parent of a gradient, but a
-///   [`grid`]($grid) will.
+/// - For other shapes, the ancestor is the innermost [`block`] or [`box`] that
+///   contains the shape. This includes the boxes and blocks that are implicitly
+///   created by show rules and elements. For example, a [`rotate`] will not
+///   affect the parent of a gradient, but a [`grid`] will.
 ///
 /// # Color spaces and interpolation
 /// Gradients can be interpolated in any color space. By default, gradients are
@@ -161,7 +160,21 @@ use crate::visualize::{Color, ColorSpace, WeightedColor};
 /// # Presets
 /// Typst predefines color maps that you can use with your gradients. See the
 /// [`color`]($color/#predefined-color-maps) documentation for more details.
-#[ty(scope)]
+///
+/// # Note on file sizes
+///
+/// Gradients can be quite large, especially if they have many stops. This is
+/// because gradients are stored as a list of colors and offsets, which can
+/// take up a lot of space. If you are concerned about file sizes, you should
+/// consider the following:
+/// - SVG gradients are currently inefficiently encoded. This will be improved
+///   in the future.
+/// - PDF gradients in the [`color.hsv`]($color.hsv), [`color.hsl`]($color.hsl),
+///   and [`color.oklch`]($color.oklch) color spaces are stored as a list of
+///   [`color.oklab`]($color.oklab) colors with extra stops in between. This
+///   avoids needing to encode these color spaces in your PDF file, but it does
+///   add extra stops to your gradient, which can increase the file size.
+#[ty(scope, cast)]
 #[derive(Clone, PartialEq, Eq, Hash)]
 pub enum Gradient {
     Linear(Arc<LinearGradient>),
@@ -694,9 +707,9 @@ impl Gradient {
 
     /// Sample the gradient at a given position.
     ///
-    /// The position is either a position along the gradient (a [ratio]($ratio)
-    /// between `{0%}` and `{100%}`) or an [angle]($angle). Any value outside
-    /// of this range will be clamped.
+    /// The position is either a position along the gradient (a [ratio] between
+    /// `{0%}` and `{100%}`) or an [angle]. Any value outside of this range will
+    /// be clamped.
     #[func]
     pub fn sample(
         &self,
@@ -1233,37 +1246,14 @@ fn sample_stops(stops: &[(Color, Ratio)], mixing_space: ColorSpace, t: f64) -> C
     if low == 0 {
         low = 1;
     }
+
     let (col_0, pos_0) = stops[low - 1];
     let (col_1, pos_1) = stops[low];
     let t = (t - pos_0.get()) / (pos_1.get() - pos_0.get());
 
-    let out = Color::mix_iter(
+    Color::mix_iter(
         [WeightedColor::new(col_0, 1.0 - t), WeightedColor::new(col_1, t)],
         mixing_space,
     )
-    .unwrap();
-
-    // Special case for handling multi-turn hue interpolation.
-    if mixing_space == ColorSpace::Hsl || mixing_space == ColorSpace::Hsv {
-        let hue_0 = col_0.to_space(mixing_space).to_vec4()[0];
-        let hue_1 = col_1.to_space(mixing_space).to_vec4()[0];
-
-        // Check if we need to interpolate over the 360° boundary.
-        if (hue_0 - hue_1).abs() > 180.0 {
-            let hue_0 = if hue_0 < hue_1 { hue_0 + 360.0 } else { hue_0 };
-            let hue_1 = if hue_1 < hue_0 { hue_1 + 360.0 } else { hue_1 };
-
-            let hue = hue_0 * (1.0 - t as f32) + hue_1 * t as f32;
-
-            if mixing_space == ColorSpace::Hsl {
-                let [_, saturation, lightness, alpha] = out.to_hsl().to_vec4();
-                return Color::Hsl(Hsl::new(hue, saturation, lightness, alpha));
-            } else if mixing_space == ColorSpace::Hsv {
-                let [_, saturation, value, alpha] = out.to_hsv().to_vec4();
-                return Color::Hsv(Hsv::new(hue, saturation, value, alpha));
-            }
-        }
-    }
-
-    out
+    .unwrap()
 }
